/*
 * Copyright (c) 2001 Network Appliance, Inc.
 * All rights reserved.
 */

#include "api.h"
#include "api_filter.h"
#ifndef COMPILE_SOLARIS
#include "banner_stripper.h"
#endif

/* GLOBALS */
int modify_control=0;
int cache_control=0;

char* 
api_options_response(const char* icap_uri)
{
#ifdef ECHO_SERVER
	/* return a pointer to the options response string */
	FILE *fp;
	char options[3000];

	memset(options, '\0', 3000);
	assert(options_str);
	memcpy(options, options_str, strlen(options_str));
	memcpy(options+strlen(options), "\r\n", 2);

	if (cinfo.verbose)
		printf("api:Sending options:\n%s", options);
	return (char*) strdup(options);
#endif
}

void
api_log_request(const char* icap_hdr)
{
	/* NO-OP */
} 

char*
api_preview_reqmod(const char* icap_hdr, 
		   const char* client_hdr, 
		   const char* post_msg_body,
		   int post_msg_body_len,
		   int preview_size)
{
#ifdef ECHO_SERVER
	char *scratch, *line;
	char str[300], *trans_id_str, *trans_id_str_end;

	char not_interested_header[] = "ICAP/1.0 204 No Content\r\n";
	int msg_size=0, trans_id_len=0, offset;
	char* not_interested_msg;
	
	assert(icap_hdr&&client_hdr&&post_msg_body);
	
	if (cinfo.modify_percent)
		return NULL;
	else {
		/* Construct 204 with IS-TAG header and Transaction ID 
		   (if it exists) */
	        memset(str, '\0', 300);
		assert(options_str);
		assert(line=(char*)strstr(options_str, "ISTAG"));
		while (*line != '\n')
			memcpy(str+strlen(str), line++, 1);

		msg_size = strlen(not_interested_header);
		msg_size += strlen(str);
		
		trans_id_str = (char*) strstr(icap_hdr, "Transaction-Id:");
		
		if (trans_id_str) {
		         trans_id_str_end = (char*) strstr(trans_id_str, "\n");     
			 scratch = trans_id_str;
			 trans_id_str_end++;
		 	 while(scratch != trans_id_str_end) {
			         trans_id_len++;
				 scratch++;
			 }
			 msg_size += trans_id_len;
		}
		
		not_interested_msg = (char*) malloc((msg_size+5)*sizeof(char));
		memset(not_interested_msg, '\0', (msg_size+5)*sizeof(char));
		memcpy(not_interested_msg, 
		 	not_interested_header,   
			strlen(not_interested_header));
		offset = strlen(not_interested_header);
		memcpy(not_interested_msg+offset, str, strlen(str));
		offset += strlen(str);
		memcpy(not_interested_msg+offset, "\r\n", 2);
		offset += 2;
		if (trans_id_len) {
			memcpy(not_interested_msg+offset,
			       trans_id_str,
			       trans_id_len);
			offset += trans_id_len;
		}
		memcpy(not_interested_msg+offset, "\r\n", 2);
		return not_interested_msg;
	}    
#endif
}

static int
api_respmod_200_or_204(const char* origin_resp_hdr, int origin_resp_body_len)
{
	int modify_me = 0;
#ifdef BANNER_STRIPPER
	modify_me = 1;
#endif
	/* For Valley Girl filtering - we can modify content-type
	   advertised as html or text only */
	if (*origin_resp_hdr && strstr(origin_resp_hdr, "Content-Type:"))
		{
			if (strstr(origin_resp_hdr,"Content-Type: text"))
				modify_me = 1;
		}
	else
		modify_me = 1;

	modify_control++;
	if (modify_me && origin_resp_body_len && 
	    ((modify_control%100) <= cinfo.modify_percent))
		return 1;
	else
		return 0;
}

char*
api_preview_respmod(const char* icap_hdr, 
		    const char* client_hdr, 
		    const char* origin_resp_hdr,
		    const char* origin_resp_body,
		    int origin_resp_body_len,
		    int preview_size)
{
#ifdef ECHO_SERVER
	FILE *fp;
	char *line, *scratch;
	char str[300], *trans_id_str, *trans_id_str_end;

	char not_interested_header[] = "ICAP/1.0 204 No Content\r\n";
	int msg_size=0, trans_id_len=0, offset;
	char* not_interested_msg;

	assert(icap_hdr&&client_hdr&&origin_resp_hdr&&origin_resp_body);

	if (api_respmod_200_or_204(origin_resp_hdr, origin_resp_body_len))
		return NULL;
	else {
		/* Construct 204 with IS-TAG header and Transaction ID 
		   (if it exists) */
		memset(str, '\0', 300);
		assert(options_str);
		assert(line=(char*)strstr(options_str, "ISTAG"));
		while (*line != '\n')
			memcpy(str+strlen(str), line++, 1);

		msg_size = strlen(not_interested_header);
		msg_size += strlen(str);
		
		trans_id_str = (char*) strstr(icap_hdr, "Transaction-Id:");
		
		if (trans_id_str) {
		         trans_id_str_end = (char*) strstr(trans_id_str, "\n");     
			 scratch = trans_id_str;
			 trans_id_str_end++;
		 	 while(scratch != trans_id_str_end) {
			         trans_id_len++;
				 scratch++;
			 }
			 msg_size += trans_id_len;
		}
		
		not_interested_msg = (char*) malloc((msg_size+5)*sizeof(char));
		memset(not_interested_msg, '\0', (msg_size+5)*sizeof(char));
		memcpy(not_interested_msg, 
		 	not_interested_header,   
			strlen(not_interested_header));
		offset = strlen(not_interested_header);
		memcpy(not_interested_msg+offset, str, strlen(str));
		offset += strlen(str);
		memcpy(not_interested_msg+offset, "\r\n", 2);
		offset += 2;
		if (trans_id_len) {
			memcpy(not_interested_msg+offset,
				trans_id_str,
				trans_id_len);
			offset += trans_id_len;
		}
		memcpy(not_interested_msg+offset, "\r\n", 2);
		return not_interested_msg;
	}     
#endif
}

static int
api_reqmod_cachable()
{
	if (((cache_control++)%100)>=cinfo.reqmod_cache_percent) 
		return 0;
	return 1;  
}

ca_t* 
api_modify_reqmod(const char* icap_hdr, 
		  const char* client_hdr, 
		  const char* post_msg_body,
		  int post_msg_body_len)
{
#ifdef ECHO_SERVER
	/* reconstruct message to echo back to client*/
	int i, msg_size[7] = {0,0,0,0,0,0,0};
	char *result, scratch[200], str[600], body_size_str[20], *line;
	FILE* fp;
	ca_t* res = (ca_t*) malloc(sizeof(ca_t));
	
	res->beg = 0;
	res->len = 0;
	
	assert(icap_hdr && client_hdr && post_msg_body); 
	
	if (cinfo.verbose) {
		printf("api: icap_hdr METHOD:\n%s", icap_hdr);
		printf("api: client_hdr:\n%s", client_hdr);
		printf("api: post_msg_body:\n%s", post_msg_body);
	}

	/* read out the 200 OK line */
	memset(str, '\0', 300);
	assert(options_str);
	assert(line=(char*)strstr(options_str, "ICAP"));
	while (*line != '\n')
		memcpy(str+strlen(str), line++, 1);
	memcpy(str+strlen(str), "\r\n", 2);

	/* XXX this MUST return the IS-TAG line */
	assert(options_str);
	assert(line=(char*)strstr(options_str, "ISTAG"));
	while (*line != '\n')
		memcpy(str+strlen(str), line++, 1);
	memcpy(str+strlen(str), "\r\n", 2);
	assert(strstr(str, "ISTAG"));

	/* Generate Cache-Control and Attribute header */
	memset(scratch, '\0', 100);
	if (api_reqmod_cachable()) {
		memcpy(scratch, "Cache-Control: max-age=360000\r\n",
		       strlen("Cache-Control: max-age=360000\r\n"));
		memcpy(scratch+strlen(scratch), "Attribute: none\r\n",
		       strlen("Attribute: none\r\n"));
	}

	/* Generate Encapsulated header */
	if (post_msg_body_len) {
		memcpy(scratch+strlen(scratch), 
		       "Encapsulated: req-hdr=0, req-body=", 
		       strlen("Encapsulated: req-hdr=0, req-body="));
		sprintf(scratch+strlen(scratch), "%d", strlen(client_hdr)); 
		memcpy(scratch+strlen(scratch), "\r\n\r\n", 4); 
	}
	else {
		memcpy(scratch+strlen(scratch), 
		       "Encapsulated: req-hdr=0\r\n\r\n", 
		       strlen("Encapsulated: req-hdr=0\r\n\r\n"));
	}		
       
	/* ICAP REQMOD response hdr: 2OO OK + ISTAG + Encapsulated + 
	   Cache-Control*/
	msg_size[0] = strlen(str);
	msg_size[1] = strlen(scratch);
        msg_size[2] = strlen(client_hdr);
	if (post_msg_body_len>0) {
		msg_size[4] = post_msg_body_len;
		sprintf(body_size_str, "%x", msg_size[4]);
		msg_size[3] = strlen(body_size_str)+2;
	}

	for(i=0; i<5; i++)
		msg_size[5] += msg_size[i];

	if (post_msg_body_len>0) {
		/* 8 = 2 "\r\n" terminate post body
		     + 5 "0\r\n\r\n" Last chunk
		     + 1 null term. */
		msg_size[6] = (msg_size[5]+8)*sizeof(char);
	} else if (post_msg_body_len) {
		/* empty post msg. body */
		/* 6 = 5 "0\r\n\r\n" Last chunk
		     + 1 null term. */
		msg_size[6] = (msg_size[5]+6)*sizeof(char);
	} else {
		/* 1 = null term. */
		msg_size[6] = (msg_size[5]+1)*sizeof(char);
	}
	result = (char*) malloc(msg_size[6]);
	memset(result, '\0', msg_size[6]);

	memcpy(result, str, msg_size[0]);
	memcpy(result+strlen(result), scratch, msg_size[1]);
	memcpy(result+strlen(result), client_hdr, msg_size[2]);
	if (post_msg_body_len>0) {
		line = result+strlen(result);
		memcpy(line, body_size_str, msg_size[3]);
		line += msg_size[3]-2;
		memcpy(line, "\r\n", 2);
		line += 2;
		memcpy(line, post_msg_body, msg_size[4]);
		line += msg_size[4];
		memcpy(line, "\r\n0\r\n\r\n", 7);
	} else if (post_msg_body_len) {
		line = result+strlen(result);
		memcpy(line, "0\r\n\r\n", 5);
	}
	
	if (cinfo.verbose)
		printf("api: icap modified post body:\n%s", result);
	res->beg = result;
	res->len = msg_size[6];
	return res;	
#endif
}

ca_t*
api_modify_respmod(const char* icap_hdr, 
		   const char* client_hdr, 
		   const char* origin_resp_hdr,
		   const char* origin_resp_body,
		   int origin_resp_body_len)
{
#ifdef ECHO_SERVER
	/* reconstruct message to echo back to client*/
	int msg_size[7] = {0,0,0,0,0,0,0};
	int i, modify_me=0, illegal_banner=0, trailers_needed=0;
	char *result, body_size_str[20], *line, str[600];
	char trailers[] = "Cache-Control: no-cache\r\nConnection: close\r\n";
	FILE *fp;
	gbuf_t* new_body;
	ca_t* res = (ca_t*) malloc(sizeof(ca_t));
	res->beg = 0;
	res->len = 0;
	
	assert(icap_hdr && client_hdr && origin_resp_hdr && origin_resp_body); 
	
	memset(str, '\0', 600);
	if (cinfo.verbose) {
		printf("api: icap_hdr METHOD:\n%s", icap_hdr);
		printf("api: client_hdr:\n%s", client_hdr);
		printf("api: origin_resp_hdr:\n%s", origin_resp_hdr);
	}
	
	/* read out the 200 OK line */
	memset(str, '\0', 300);
	assert(options_str);
	assert(line=(char*)strstr(options_str, "ICAP"));
	while (*line != '\n')
		memcpy(str+strlen(str), line++, 1);
	memcpy(str+strlen(str), "\r\n", 2);

	/* XXX this MUST return the IS-TAG line */
	assert(options_str);
	assert(line=(char*)strstr(options_str, "ISTAG"));
	while (*line != '\n')
		memcpy(str+strlen(str), line++, 1);
	memcpy(str+strlen(str), "\r\n", 2);
	assert(strstr(str, "ISTAG"));

	/* Insert Encapsulated header */
	if (*origin_resp_hdr) {
		memcpy(str+strlen(str), "Encapsulated: res-hdr=0, res-body=",
		       strlen("Encapsulated: res-hdr=0, res-body="));
		sprintf(str+strlen(str), "%d", strlen(origin_resp_hdr));
		memcpy(str+strlen(str), "\r\n\r\n", 4);
	} 
	else if (*origin_resp_body) {
	        memcpy(str+strlen(str), "Encapsulated: res-body=0",
		       strlen("Encapsulated: res-body=0"));
		memcpy(str+strlen(str), "\r\n\r\n", 4);
	}
	else
		memcpy(str+strlen(str), "\r\n", 2);

	msg_size[0] = strlen(str);
	msg_size[2] = strlen(origin_resp_hdr);

#ifndef WINDOWS	
#ifdef FILTER
	if (strstr(origin_resp_hdr, "Content-Type:")&&
		origin_resp_body_len)
		{
			if (strstr(origin_resp_hdr,"Content-Type: text/html")||
			    strstr(origin_resp_hdr,"Content-Type: text/text")){
				modify_me = 1;
				new_body = api_filter(origin_resp_body, 
						      origin_resp_body_len);
				origin_resp_body_len = new_body->length;
				origin_resp_body = new_body->data;
				free(new_body);
			}
		}
#endif

#ifdef BANNER_STRIPPER
	if (origin_resp_body_len &&
	    *origin_resp_body == 'G' && 
	    *(origin_resp_body+1) == 'I' && 
	    *(origin_resp_body+2) == 'F') {
		
		new_body = strip_banner(origin_resp_body,
					origin_resp_body_len);
		
		if (new_body->length) {
			illegal_banner=1;
			origin_resp_body_len = new_body->length;
			origin_resp_body = new_body->data;
		}
		else 
		        free(new_body->data);
		free(new_body);
	}
#endif
#endif
	/* handle case in which body size=0 */
	msg_size[4] = origin_resp_body_len;
	sprintf(body_size_str, "%x", msg_size[4]);
	msg_size[3] = strlen(body_size_str);
		
	for(i=0; i<5; i++)
		msg_size[5] += msg_size[i];
        
	/* determine if trailers need to be added */
	if (cinfo.trailer_support) {
		if (strstr(icap_hdr, "TE")) {
			trailers_needed = 1;
		}
	}

        /* handle case in which body size=0 */
	if (origin_resp_body_len) {
		/* 10 = 2 (\r\n for body_size_str term.)
		      + 2 (\r\n for terminating actual last data chunk)
		      + 5 (0\r\n\r\n for last 0 byte chunk)
		      + 1 (NULL term. for string)
		*/
		msg_size[6] = (msg_size[5]+10)*sizeof(char) +
			trailers_needed*strlen(trailers);
	}
	else {
		/* 5 = 2 (\r\n for body_size_str term.)
		     + 2 (\r\n for last 0 byte chunk)
		     + 1 (NULL term. for string */
		msg_size[6] = (msg_size[5]+5)*sizeof(char) +
			trailers_needed*strlen(trailers);
	}
	result = (char*) malloc(msg_size[6]);
	memset(result, '\0', msg_size[6]);
	
	memcpy(result, str, msg_size[0]);
	i = msg_size[0];
	
	if (*origin_resp_hdr) {
		memcpy(result+i, origin_resp_hdr, msg_size[2]);
		i += msg_size[2];
	}
	
	memcpy(result+i, body_size_str, msg_size[3]);
	i += msg_size[3];
	memcpy(result+i, "\r\n", 2);
	i += 2;
	
	/* handle case in which body size=0 */
	if (origin_resp_body_len) {
		memcpy(result+i, origin_resp_body, msg_size[4]);
		i += msg_size[4];
		if (trailers_needed) {
			memcpy(result+i, "\r\n0\r\n", 5);
			i+= 5;
			memcpy(result+i, trailers, strlen(trailers));
			i += strlen(trailers);
			memcpy(result+i, "\r\n", 2);
			i+= 2;
		}
		else
			memcpy(result+i, "\r\n0\r\n\r\n", 7);
	}
	else {
		if (trailers_needed) {
			memcpy(result+i, trailers, strlen(trailers));
			i += strlen(trailers);
			memcpy(result+i, "\r\n", 2);
			i+= 2;
		} 
		else
			memcpy(result+i, "\r\n", 2);
	}

#ifndef WINDOWS
#ifdef FILTER
	if (modify_me)
		free(origin_resp_body);
#endif
#ifdef BANNER_STRIPPER
	if (illegal_banner)
		free(origin_resp_body);
#endif
#endif /* WINDOWS */	

	res->beg = result;
	res->len = msg_size[6];
	return res;
#endif /* ECHO_SERVER */

}
