/*
 * Copyright (c) 2001 Network Appliance, Inc.
 * All rights reserved.
 */

#ifndef _API_H_
#define _API_H_

#include <stdio.h>

/* The following #define must be #undef if the API is to be utilized 
 * by a Netcache ICAP partner using proprietary api calls
 */
#define ECHO_SERVER
/* #define FILTER */

#ifdef ECHO_SERVER
#include "iserver.h"
#include "assert.h"
#include "gbuf.h"

#ifdef WINDOWS
#include <string.h>
#include <Windows.h>
HANDLE file_mtx;
#else
#include <strings.h>
pthread_mutex_t file_mtx;
#endif

#endif

/* return an OPTIONS response depending on the ICAP service requested.
 * Caller of this routine will free the memory allocated by char*
 */
extern char* 
api_options_response(const char* icap_uri);

/* process the LOG request sent by client */
extern void
api_log_request(const char* icap_hdr); 

/*
 * return response for preview received
 * return NULL if 100 Continue (no headers are allowed with a 100 Continue)
 * is to be sent else construct message for 204 and return allocated char*.
 * Calling routine will free mem 
 */
extern char*
api_preview_reqmod(const char* icap_hdr, 
		   const char* client_hdr, 
		   const char* post_msg_body,
		   int post_msg_body_len,
		   int preview_size);
extern char*
api_preview_respmod(const char* icap_hdr, 
		    const char* client_hdr, 
		    const char* origin_resp_hdr,
		    const char* origin_resp_body,
		    int origin_resp_body_len,
		    int preview_size);

/* send modified headers and message body in char* returned. Caller
 * of this routine will free the memory allocated by char*.
 */
extern ca_t* 
api_modify_reqmod(const char* icap_hdr, 
		  const char* client_hdr,
		  const char* post_msg_body,
		  int post_msg_body_len);

extern ca_t* 
api_modify_respmod(const char* icap_hdr, 
		   const char* client_hdr, 
		   const char* origin_resp_hdr,
		   const char* origin_resp_body,
		   int origin_resp_body_len);

#endif // _API_H_
